package com.menagerie.bakers

import androidx.compose.animation.ExperimentalSharedTransitionApi
import androidx.compose.animation.SharedTransitionLayout
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.lazy.grid.GridCells
import androidx.compose.foundation.lazy.grid.LazyVerticalGrid
import androidx.compose.foundation.lazy.grid.rememberLazyGridState
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.automirrored.filled.ExitToApp
import androidx.compose.material.icons.filled.Settings
import androidx.compose.material3.Button
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateListOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import bakersmenagerie.composeapp.generated.resources.Res
import bakersmenagerie.composeapp.generated.resources.auto_mode
import bakersmenagerie.composeapp.generated.resources.dark_mode
import bakersmenagerie.composeapp.generated.resources.history
import bakersmenagerie.composeapp.generated.resources.history_android
import bakersmenagerie.composeapp.generated.resources.light_mode
import bakersmenagerie.composeapp.generated.resources.shuffle
import bakersmenagerie.composeapp.generated.resources.tune
import com.menagerie.bakers.model.Recipe
import com.menagerie.bakers.model.TagType
import com.menagerie.bakers.ui.theme.MainTheme
import com.menagerie.bakers.view.BookShelf
import com.menagerie.bakers.view.HomeScreen
import com.menagerie.bakers.view.SettingsMenu
import com.menagerie.bakers.view.details.RecipeDetails
import com.menagerie.bakers.view.list.RecipesListScreen
import com.menagerie.bakers.view.util.FilterCard
import com.menagerie.bakers.view.util.InputFieldState
import com.menagerie.bakers.view.util.getFilteredRecipeList
import com.menagerie.bakers.view.util.getRecipeList
import com.russhwolf.settings.Settings
import org.jetbrains.compose.resources.painterResource


enum class Theme {
    Auto,
    Light,
    Dark
}

enum class SortBy {
    Name,
    Time,
    Ingredients,
}

enum class RecipeAppScreen {
    List,
    History,
    Details,
    DetailHistory,
}

@OptIn(ExperimentalSharedTransitionApi::class)
@Composable
fun App(
    onClose: () -> Unit = {}
) {

    //Grab an instance of the Settings object
    val settings by remember { mutableStateOf(Settings()) }

    //used to navigate through the different compose screens
    val navController = rememberNavController()

    //Grab and set Theme from Settings
    val theme = SettingsHolder.theme.value

    val isDarkTheme: Boolean? = when (theme) {
        Theme.Dark -> true
        Theme.Light -> false
        Theme.Auto -> null
    }
    // END Theme


    MainTheme(useDarkTheme = isDarkTheme ?: isSystemInDarkTheme()) {


        var showFilter by remember { mutableStateOf(false) }
        var menuBar by remember { mutableStateOf(false) }
        var searchBar by remember { mutableStateOf(false) }
        var history by remember { mutableStateOf(false) }
        var search by remember { mutableStateOf("") }
        val tags = remember { mutableStateListOf<String>() }
        val recipeTags by remember { mutableStateOf(mutableMapOf<String, TagType>()) }
        var book by remember { mutableStateOf("") }
        val remove = remember { mutableStateListOf<String>() }

        val histogram = remember { mutableListOf<Recipe>() }

        val filteredItems = getFilteredRecipeList(
            tags = tags,
            search = search,
            lockTag = book,
        )

        recipeTags.clear()

        for (recipe in filteredItems) {
            for (tag in recipe.tags) {
                if (!recipeTags.contains(tag.key))
                    recipeTags[tag.key] = tag.value
            }
        }

        if (showFilter) {
            FilterCard(
                recipeTags = recipeTags,
                activeTags = tags,
            ) {
                if (it.isEmpty()) {
                    tags.clear()
                }

                for (item in it)
                    if (tags.contains(item).not())
                        tags.add(item)

                remove.clear()

                for (item in tags)
                    if (it.contains(item).not())
                        remove.add(item)

                tags.removeAll(remove)

                showFilter = false
            }
        }


        val recipeCount = getRecipeList().size
        var currentRecipe: Recipe

        if (filteredItems.isNotEmpty())
            currentRecipe = filteredItems.first()

        Scaffold {
            Column {
                when (DisplayManager.device.value) {
                    Device.Android ->
                        if (menuBar)
                            Column {
                                Row {
                                    SettingsMenu(
                                        onRandom = {
                                            currentRecipe = filteredItems.random()
                                            navController.navigate(
                                                RecipeAppScreen.Details.name.plus(
                                                    "/${currentRecipe.title}"
                                                )
                                            )
                                        },
                                        onBack = {},
                                        onClear = {settings.clear()},
                                    )

                                    Spacer(modifier = Modifier.weight(1f))

                                    if(searchBar)
                                    InputFieldState(
                                        value = search,
                                        label = "Search ".plus(book),
                                        modifier = Modifier
                                            .width(if (DisplayManager.size.value == Size.Large || DisplayManager.orientation.value == Orientation.Landscape) 400.dp else 250.dp)
                                            .align(Alignment.CenterVertically)
                                    ) {
                                        search = it
                                    }

                                    if (DisplayManager.orientation.value == Orientation.Portrait)
                                        Spacer(modifier = Modifier.weight(1f))

                                    if(histogram.isNotEmpty() && searchBar)
                                    {
                                        IconButton(
                                            onClick = {
                                                navController.navigate(RecipeAppScreen.History.name)
                                            },
                                        ) {
                                            Icon(painter = painterResource(Res.drawable.history_android), "History")
                                        }
                                    }

                                    if(searchBar)
                                    IconButton(
                                        onClick = { showFilter = true },
                                    ) {
                                        Icon(
                                            painter = painterResource(Res.drawable.tune), "Filter"
                                        )
                                    }
                                }
                                val listState = rememberLazyGridState()

                                if(searchBar)
                                LazyVerticalGrid(
                                    state = listState,
                                    columns = GridCells.Adaptive(minSize = 125.dp)
                                )
                                {
                                    items(tags.size) {
                                        Button(onClick = { tags.remove(tags[it]) }) {
                                            Text(text = tags[it])
                                        }
                                    }
                                }
                            }

                    Device.Desktop ->
                        if (menuBar)
                            Column {
                                Row {
                                    IconButton(
                                        onClick = { navController.popBackStack() },
                                        content = {
                                            Icon(
                                                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                                                contentDescription = "Back"
                                            )
                                        }
                                    )
                                    IconButton(
                                        onClick = { navController.navigate("Settings") },
                                        content = {
                                            Icon(
                                                imageVector = Icons.Default.Settings,
                                                contentDescription = ""
                                            )
                                        }
                                    )
                                    IconButton(
                                        onClick = {SettingsHolder.setTheme()},
                                        content = {
                                            Icon(
                                                painter = painterResource(
                                                    when(theme) {
                                                        Theme.Auto -> Res.drawable.auto_mode
                                                        Theme.Light -> Res.drawable.light_mode
                                                        Theme.Dark -> Res.drawable.dark_mode
                                                    }
                                                ),
                                                contentDescription = "theme"
                                            )
                                        }
                                    )

                                    if(history)
                                    IconButton(
                                            onClick = {
                                                currentRecipe = histogram.random()
                                                navController.navigate(
                                                    RecipeAppScreen.DetailHistory.name.plus(
                                                        "/${currentRecipe.title}"
                                                    )
                                                )
                                            },
                                    content = {
                                        Icon(
                                            painter = painterResource(Res.drawable.shuffle),
                                            contentDescription = "random"
                                        )
                                    }
                                    )
                                    else
                                    IconButton(
                                        onClick = {
                                            currentRecipe = filteredItems.random()
                                            navController.navigate(
                                                RecipeAppScreen.Details.name.plus(
                                                    "/${currentRecipe.title}"
                                                )
                                            )
                                        },
                                        content = {
                                            Icon(
                                                painter = painterResource(Res.drawable.shuffle),
                                                contentDescription = "random"
                                            )
                                        }
                                    )

                                    Spacer(modifier = Modifier.weight(1f))

                                    if(searchBar)
                                    InputFieldState(
                                        value = search,
                                        label = "Search ".plus(book),
                                        modifier = Modifier
                                            .width(if (DisplayManager.size.value == Size.Large || DisplayManager.orientation.value == Orientation.Landscape) 400.dp else 250.dp)
                                            .align(Alignment.CenterVertically)
                                    ) {
                                        search = it
                                    }

                                    if (DisplayManager.orientation.value == Orientation.Portrait)
                                        Spacer(modifier = Modifier.weight(1f))

                                    if(histogram.isNotEmpty() && searchBar)
                                    {
                                        IconButton(
                                            onClick = {
                                                navController.navigate(RecipeAppScreen.History.name)
                                            },
                                            modifier = Modifier.align(Alignment.CenterVertically).padding(bottom = 8.dp)
                                        ) {
                                            Icon(painter = painterResource(Res.drawable.history), "History")
                                        }
                                    }

                                    if(searchBar)
                                    IconButton(
                                        onClick = { showFilter = true },
                                        modifier = Modifier.align(Alignment.CenterVertically).padding(bottom = 8.dp)
                                    ) {
                                        Icon(
                                            painter = painterResource(Res.drawable.tune), "Filter",
                                        )
                                    }
                                    IconButton(
                                        onClick = onClose,
                                        modifier = Modifier.align(Alignment.CenterVertically).padding(bottom = 8.dp)
                                    ) {
                                        Icon(
                                            imageVector = Icons.AutoMirrored.Filled.ExitToApp, "Close",
                                        )
                                    }
                                }
                                if(searchBar)
                                Row {
                                    Spacer(modifier = Modifier.weight(1f))
                                    if (DisplayManager.orientation.value == Orientation.Landscape && (tags.isNotEmpty() || search.isNotEmpty() || book.isNotEmpty()))
                                        Text(
                                            text = "Showing ".plus(filteredItems.size)
                                                .plus(" out of ")
                                                .plus(recipeCount).plus(" recipes"),
                                        )
                                }
                                val listState = rememberLazyGridState()
                                if(searchBar)
                                LazyVerticalGrid(
                                    state = listState,
                                    columns = GridCells.Adaptive(minSize = 125.dp)
                                )
                                {
                                    items(tags.size) {
                                        Button(onClick = { tags.remove(tags[it]) }) {
                                            Text(text = tags[it])
                                        }
                                    }
                                }
                            }

                    Device.Web ->
                        if(menuBar)
                        Column {
                            Row {
                                IconButton(
                                    onClick = { navController.popBackStack() },
                                    content = {
                                        Icon(
                                            imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                                            contentDescription = "Back"
                                        )
                                    }
                                )
                                IconButton(
                                    onClick = {
                                        currentRecipe = filteredItems.random()
                                        navController.navigate(
                                            RecipeAppScreen.Details.name.plus(
                                                "/${currentRecipe.title}"
                                            )
                                        )
                                    },
                                    content = {
                                        Icon(
                                            painter = painterResource(Res.drawable.shuffle),
                                            contentDescription = "random"
                                        )
                                    }
                                )
                                IconButton(
                                    onClick = {SettingsHolder.setTheme()},
                                    content = {
                                        Icon(
                                            painter = painterResource(
                                                when(theme) {
                                                    Theme.Auto -> Res.drawable.auto_mode
                                                    Theme.Light -> Res.drawable.light_mode
                                                    Theme.Dark -> Res.drawable.dark_mode
                                                }
                                            ),
                                            contentDescription = "theme"
                                        )
                                    }
                                )
                            }
                        }
                }


                SharedTransitionLayout {
                    NavHost(
                        navController = navController,
                        startDestination = "Home",
                        modifier = Modifier.fillMaxSize()
                    ) {
                        composable(route = "Home")
                        {
                            menuBar = false
                            HomeScreen(
                                onClose = onClose,
                                onGo = {
                                    if (DisplayManager.device.value == Device.Web)
                                        navController.navigate(RecipeAppScreen.List.name)
                                    else
                                        navController.navigate("BookShelf")
                                },
                                onSurprise = {
                                    currentRecipe = filteredItems.random()
                                    navController.navigate(RecipeAppScreen.Details.name.plus("/${currentRecipe.title}"))
                                },
                            )
                        }
                        composable(route = "BookShelf")
                        {
                            val shelfTags = mutableMapOf<String, TagType>()

                            for (recipe in getRecipeList()) {
                                for (tag in recipe.tags) {
                                    if (!tags.contains(tag.key))
                                        shelfTags[tag.key] = tag.value
                                }
                            }

                            menuBar = false
                            history = false
                            BookShelf(
                                tags = shelfTags,
                                onClick = { lockedTag ->

                                    tags.clear()
                                    search = ""

                                    book = lockedTag
                                    if (book == "SURPRISE") {
                                        currentRecipe = filteredItems.random()
                                        navController.navigate(RecipeAppScreen.Details.name.plus("/${currentRecipe.title}"))
                                    } else {
                                        navController.navigate(RecipeAppScreen.List.name)
                                    }
                                }
                            )
                        }
                        composable(route = RecipeAppScreen.List.name) {
                            menuBar = true
                            searchBar = true
                            history = false

                            RecipesListScreen(
                                items = filteredItems,
                                onClick = { recipe ->
                                    currentRecipe = recipe
                                    navController.navigate(RecipeAppScreen.Details.name.plus("/${recipe.title}"))
                                })
                        }

                        composable(route = RecipeAppScreen.History.name) {
                            menuBar = true
                            searchBar = false
                            history = true

                            RecipesListScreen(
                                items = histogram,
                                onClick = { recipe ->
                                    currentRecipe = recipe
                                    navController.navigate(RecipeAppScreen.DetailHistory.name.plus("/${recipe.title}"))
                                })
                        }

                        composable(
                            route = RecipeAppScreen.Details.name.plus("/{recipe}"),
                            arguments = listOf(navArgument("recipe") {
                                type = NavType.StringType
                            })
                        ) { args ->
                            menuBar = false
                            history = false
                            getRecipeList().find { it.title == args.arguments?.getString("recipe") }
                                ?.let { it1 ->

                                    if(histogram.contains(it1).not())
                                        histogram.add(it1)

                                    RecipeDetails(
                                        navController = navController,
                                        recipe = it1,
                                        multipleRecipes = filteredItems.size > 1,
                                        navTo = RecipeAppScreen.List.name,
                                        goForward = {

                                            val index = filteredItems.indexOf(it1)
                                            val recipe =
                                                if (index < filteredItems.size - 1) filteredItems[index + 1] else filteredItems.first()

                                            navController.navigate(
                                                RecipeAppScreen.Details.name.plus(
                                                    "/${recipe.title}"
                                                )
                                            )
                                        })
                                }
                        }
                        composable(
                            route = RecipeAppScreen.DetailHistory.name.plus("/{recipe}"),
                            arguments = listOf(navArgument("recipe") {
                                type = NavType.StringType
                            })
                        ) { args ->
                            menuBar = false
                            history = true
                            getRecipeList().find { it.title == args.arguments?.getString("recipe") }
                                ?.let { it1 ->

                                    RecipeDetails(
                                        navController = navController,
                                        recipe = it1,
                                        multipleRecipes = histogram.size > 1,
                                        navTo = RecipeAppScreen.History.name,
                                        goForward = {

                                            val index = histogram.indexOf(it1)
                                            val recipe =
                                                if (index < histogram.size - 1) histogram[index + 1] else histogram.first()

                                            navController.navigate(
                                                RecipeAppScreen.DetailHistory.name.plus(
                                                    "/${recipe.title}"
                                                )
                                            )
                                        })
                                }
                        }

                        composable(
                            route = "Settings"
                        ) {
                            menuBar = false
                            history = false
                            SettingsMenu(
                                onRandom = {},
                                onBack = { navController.popBackStack() },
                                onClear = {settings.clear()},
                            )
                        }
                    }
                }
            }
        }
    }
}